/*
 * This Java source file was generated by the Gradle 'init' task.
 */
package HpdosClient;

import HpdosClient.MessageFormat.MessageConstants;
import HpdosClient.MessageFormat.RequestBuilder;
import hpdos.grpc.NetworkServiceGrpc;
import hpdos.grpc.Packet;
import hpdos.grpc.PacketFormatProto;
import hpdos.grpc.Request;
import io.grpc.ManagedChannel;
import io.grpc.ManagedChannelBuilder;

import java.sql.Timestamp;
import java.time.Instant;
import java.util.*;
import java.util.concurrent.*;

public class ClientRunner {
    public static final int parallelCount = 10;
    private final String clientID;

    public ClientRunner() {
        clientID = UUID.randomUUID().toString();
    }
    public String getGreeting() {
        return "Hello World!";
    }

    // create a metadata block
    public long create(NetworkServiceGrpc.NetworkServiceBlockingStub stub, String key, String value) {
        ArrayList<Request> request = new ArrayList<>();
        request.add(RequestBuilder.buildRequest(MessageConstants.METADATA_CREATE,
                0, value.length(), key,
                0, MessageConstants.METADATA_ACCESS_PRIVATE, this.clientID, value));
        Packet packet = RequestBuilder.buildPacket(request);
        long timestampCreateStart = System.currentTimeMillis();
        Packet response = stub.createMetadata(packet);
        return timestampCreateStart;
    }

    // read back the metadata
    public Map.Entry<Packet, Long> read(NetworkServiceGrpc.NetworkServiceBlockingStub stub, String key) {
        ArrayList<Request> request = new ArrayList<>();
        request.add(RequestBuilder.buildRequest(MessageConstants.METADATA_READ,
                0, 0, key, 0, MessageConstants.METADATA_ACCESS_PRIVATE, this.clientID, ""));
        Packet packet = RequestBuilder.buildPacket(request);
        long timestampReadStart = System.currentTimeMillis();
        Packet response = stub.readMetadata(packet);
//        System.out.println(response);
        return new AbstractMap.SimpleEntry<>(response, timestampReadStart);
    }

    public long update(NetworkServiceGrpc.NetworkServiceBlockingStub stub, String key, String value, int version) {
        ArrayList<Request> request = new ArrayList<>();
        request.add(RequestBuilder.buildRequest(MessageConstants.METADATA_UPDATE,
                version, value.length(), key,
                0, MessageConstants.METADATA_ACCESS_PRIVATE, this.clientID, value));
        Packet packet = RequestBuilder.buildPacket(request);
        long timestampCreateStart = System.currentTimeMillis();
        Packet response = stub.updateMetadata(packet);
        return timestampCreateStart;
    }

    public long delete(NetworkServiceGrpc.NetworkServiceBlockingStub stub, String key, int version) {
        ArrayList<Request> request = new ArrayList<>();
        request.add(RequestBuilder.buildRequest(MessageConstants.METADATA_DELETE,
                version, 0, key,
                0, MessageConstants.METADATA_ACCESS_PRIVATE, this.clientID, ""));
        Packet packet = RequestBuilder.buildPacket(request);
        long timestampCreateStart = System.currentTimeMillis();
        Packet response = stub.deleteMetadata(packet);
        return timestampCreateStart;
    }

    public double initClient(String id) {
        final ManagedChannel channel = ManagedChannelBuilder.
                forAddress(ConfigConstants.HOST, ConfigConstants.PORT)
                .usePlaintext()
                .build();
        double averageReadTime = 0, averageWriteTime = 0, averageUpdateTime = 0, averageDeleteTime = 0;
        long startTime = System.currentTimeMillis();
        for (int i = 1; i <= 1000; i++) {

            String key = id + (int) (Math.random() * Integer.MAX_VALUE),
                    value = "dummy",
                    updatedValue = "dummyUpdated";
            NetworkServiceGrpc.NetworkServiceBlockingStub stub = NetworkServiceGrpc.newBlockingStub(channel);
            long timestampCreateStart = create(stub, key, value);
            averageWriteTime = (averageWriteTime * (i - 1) + (System.currentTimeMillis() - timestampCreateStart)) / i;
            AbstractMap.Entry<Packet, Long> data = read(stub, key);
            averageReadTime = (averageReadTime * (i - 1) + (System.currentTimeMillis() - data.getValue())) / i;
            long timestampUpdateStart = update(stub, key, updatedValue, data.getKey().getResponse(0).getAck().getVersion());
            averageUpdateTime = (averageUpdateTime * (i - 1) + (System.currentTimeMillis() - timestampUpdateStart)) / i;
            data = read(stub, key);
            averageReadTime = (averageReadTime * (i - 1) + (System.currentTimeMillis() - data.getValue())) / i;
            long timestampDeleteStart = delete(stub, key, data.getKey().getResponse(0).getAck().getVersion());
            averageDeleteTime = (averageDeleteTime * (i - 1) + (System.currentTimeMillis() - timestampDeleteStart)) / i;
            data = read(stub, key);
            averageReadTime = (averageReadTime * (i - 1) + (System.currentTimeMillis() - data.getValue())) / i;

            if (i % 1000 == 0)
                System.out.println(id + "Avg MDS time read: "
                    + averageReadTime
                    + " ms\t create: " + averageWriteTime
                    + " ms\t update: " + averageUpdateTime
                    + " ms\t delete: " + averageDeleteTime + "ms"
                );
        }
        double qps = (1000 * 1000 / (System.currentTimeMillis() * 1.0 - startTime));
//        System.out.println(id + "runtime " + (System.currentTimeMillis() - startTime) +
//                "ms qps " + qps);
        channel.shutdown();
        return qps;
    }


    public static void main(String[] args) throws InterruptedException, ExecutionException {
        ClientRunner clientRunner = new ClientRunner();
        System.out.println(clientRunner.getGreeting());
        Thread.sleep(1000);
        ExecutorService executorService = Executors.newFixedThreadPool(parallelCount);
        Set<Callable<Double>> callables = new HashSet<>();
        for (int i = 0; i < parallelCount; i++) {
            int finalI = i;
            callables.add(() -> clientRunner.initClient(Integer.toString(finalI)));
        }

        long startTime = System.currentTimeMillis();
        List<Future<Double>> futures = executorService.invokeAll(callables);
        double qps = 0;
        for (Future<Double> future: futures) {
            qps += future.get();
        }
        long endTime = System.currentTimeMillis();
        System.out.println();
        System.out.println("QPS Read: " + 3 * qps + " Write: " + qps + " Update: " + qps + " Delete: " + qps);
        System.out.println("QPS Total: " + 6 * qps + "avg query time: " + ((6 * qps) / (endTime - startTime)));
//        clientRunner.initClient();
    }
}
